// hlavickove subory *******************************************************
#include "../includes.h"
// vlastny .h subor
#include "water.h"

// konstruktor *************************************************************
CWater::CWater()
{
 // identifikatory textur
 tex_nm=tex_dudv=tex_reflection=tex_refraction=tex_depth=0;
}

// destruktor **************************************************************
CWater::~CWater()
{
 glDeleteBuffers(1,&v_buf);
 glDeleteBuffers(1,&t_buf);
 glDeleteBuffers(1,&tbn_t_buf);
 glDeleteBuffers(1,&tbn_b_buf);
 glDeleteBuffers(1,&tbn_n_buf);

 // Vertex Array Object
 glDeleteVertexArrays(1,&vao);
}

// inicializacia objektu predstavujuceho hladinu vody **********************
int CWater::init(void)
{
 tex_nm=100; tex_dudv=101;
 tex_reflection=102; tex_refraction=103; tex_depth=104;

 // nahratie normalmap textury pre vodu
 if (texture.load2D(tex_nm,"data/textures/water_nm.jpg",T_LINEAR | T_MIPMAP)) {
  printf("Chyba nahratia normalmap textury pre vodu!\n"); return(1);
 }
 // nahratie dudv textury pre vodu
 if (texture.load2D(tex_dudv,"data/textures/water_dudv.jpg",T_LINEAR | T_MIPMAP)) {
  printf("Chyba nahratia dudv textury pre vodu!\n"); return(1);
 }
 // nahratie reflection textury pre vodu
 if (texture.load2D(tex_reflection,"data/textures/water_rfl.jpg",T_LINEAR)) {
  printf("Chyba nahratia reflection textury pre vodu!\n"); return(1);
 }
 // nahratie refraction textury pre vodu
 if (texture.load2D(tex_refraction,"data/textures/water_rfl.jpg",T_LINEAR)) {
  printf("Chyba nahratia refraction textury pre vodu!\n"); return(1);
 }
 // vytvorenie depth textury pre vodu
 if (texture.create_grayscale(tex_depth)) {
  printf("Chyba vytvorenia depth textury pre vodu!\n"); return(1);
 }

 calc_water_plane(0.0f,1.0f,0.0f,1.0f);

 loopi(18) v[i]=water_level;
 v[0]=-5.0f; v[2]=5.0f; v[3]=5.0f; v[5]=5.0f; v[6]=5.0f; v[8]=-5.0f;
 v[9]=-5.0f; v[11]=5.0f; v[12]=5.0f; v[14]=-5.0f; v[15]=-5.0f; v[17]=-5.0f;
 t[0]=0.0f; t[1]=0.0f; t[2]=1.0f; t[3]=0.0f; t[4]=1.0f; t[5]=1.0f;
 t[6]=0.0f; t[7]=0.0f; t[8]=1.0f; t[9]=1.0f; t[10]=0.0f; t[11]=1.0f;
 // vypocet TBN
 vec3 v0,v1,v2;
 vec2 t0,t1,t2;
 // pre 1. trojuholnik
 //           . v2
 //         . .
 //       .   .
 //     .     .
 // v0 ........ v1
 v0.set(v[0],v[1],v[2]); v1.set(v[3],v[4],v[5]); v2.set(v[6],v[7],v[8]);
 t0.set(t[0],t[1]); t1.set(t[2],t[3]); t2.set(t[4],t[5]);
 mm.calc_tbn(v0,v1,v2,t0,t1,t2,tan,bit,nor); // vypocet TBN
 tbn_t[0]=tan.x; tbn_t[1]=tan.y; tbn_t[2]=tan.z;
 tbn_t[3]=tan.x; tbn_t[4]=tan.y; tbn_t[5]=tan.z;
 tbn_t[6]=tan.x; tbn_t[7]=tan.y; tbn_t[8]=tan.z;
 tbn_b[0]=bit.x; tbn_b[1]=bit.y; tbn_b[2]=bit.z;
 tbn_b[3]=bit.x; tbn_b[4]=bit.y; tbn_b[5]=bit.z;
 tbn_b[6]=bit.x; tbn_b[7]=bit.y; tbn_b[8]=bit.z;
 tbn_n[0]=nor.x; tbn_n[1]=nor.y; tbn_n[2]=nor.z;
 tbn_n[3]=nor.x; tbn_n[4]=nor.y; tbn_n[5]=nor.z;
 tbn_n[6]=nor.x; tbn_n[7]=nor.y; tbn_n[8]=nor.z;
 // pre 2. trojuholnik
 // v2 ........ v1
 //    .     .
 //    .   .
 //    . .
 // v0 .
 v0.set(v[9],v[10],v[11]); v1.set(v[12],v[13],v[14]); v2.set(v[15],v[16],v[17]);
 t0.set(t[6],t[7]); t1.set(t[8],t[9]); t2.set(t[10],t[11]);
 mm.calc_tbn(v0,v1,v2,t0,t1,t2,tan,bit,nor); // vypocet TBN
 tbn_t[9]=tan.x; tbn_t[10]=tan.y; tbn_t[11]=tan.z;
 tbn_t[12]=tan.x; tbn_t[13]=tan.y; tbn_t[14]=tan.z;
 tbn_t[15]=tan.x; tbn_t[16]=tan.y; tbn_t[17]=tan.z;
 tbn_b[9]=bit.x; tbn_b[10]=bit.y; tbn_b[11]=bit.z;
 tbn_b[12]=bit.x; tbn_b[13]=bit.y; tbn_b[14]=bit.z;
 tbn_b[15]=bit.x; tbn_b[16]=bit.y; tbn_b[17]=bit.z;
 tbn_n[9]=nor.x; tbn_n[10]=nor.y; tbn_n[11]=nor.z;
 tbn_n[12]=nor.x; tbn_n[13]=nor.y; tbn_n[14]=nor.z;
 tbn_n[15]=nor.x; tbn_n[16]=nor.y; tbn_n[17]=nor.z;

 // Vertex Array Object
 glGenVertexArrays(1,&vao);
 glBindVertexArray(vao);

 glGenBuffers(1,&v_buf);
 glBindBuffer(GL_ARRAY_BUFFER,v_buf);
 glBufferData(GL_ARRAY_BUFFER,sizeof(v),v,GL_STATIC_DRAW);
 // vrcholy
 glVertexAttribPointer(0,3,GL_FLOAT,GL_FALSE,0,0);
 glGenBuffers(1,&t_buf);
 glBindBuffer(GL_ARRAY_BUFFER,t_buf);
 glBufferData(GL_ARRAY_BUFFER,sizeof(t),t,GL_STATIC_DRAW);
 // texturovacie suradnice
 glVertexAttribPointer(8,2,GL_FLOAT,GL_FALSE,0,0);
 // T
 glGenBuffers(1,&tbn_t_buf);
 glBindBuffer(GL_ARRAY_BUFFER,tbn_t_buf);
 glBufferData(GL_ARRAY_BUFFER,sizeof(tbn_t),tbn_t,GL_STATIC_DRAW);
 glVertexAttribPointer(2,3,GL_FLOAT,GL_FALSE,0,0);
 // B
 glGenBuffers(1,&tbn_b_buf);
 glBindBuffer(GL_ARRAY_BUFFER,tbn_b_buf);
 glBufferData(GL_ARRAY_BUFFER,sizeof(tbn_b),tbn_b,GL_STATIC_DRAW);
 glVertexAttribPointer(3,3,GL_FLOAT,GL_FALSE,0,0);
 // N
 glGenBuffers(1,&tbn_n_buf);
 glBindBuffer(GL_ARRAY_BUFFER,tbn_n_buf);
 glBufferData(GL_ARRAY_BUFFER,sizeof(tbn_n),tbn_n,GL_STATIC_DRAW);
 glVertexAttribPointer(4,3,GL_FLOAT,GL_FALSE,0,0);

 glEnableVertexAttribArray(0);
 glEnableVertexAttribArray(2);
 glEnableVertexAttribArray(3);
 glEnableVertexAttribArray(4);
 glEnableVertexAttribArray(8);

 glBindVertexArray(0);

 return(0);
}

// vykreslenie objektu predstavujuceho hladinu vody ************************
void CWater::draw(void)
{
 glBindVertexArray(vao);
 glDrawArrays(GL_TRIANGLES,0,6);
 glBindVertexArray(0);
}
